@@ -166,8 +166,14 @@ static void php_gnupg_this_make(PHPC_THIS_DECLARE(gnupg), zval *options TSRMLS_D
 		}
 
 		if (file_name != NULL || home_dir != NULL) {
-			gpgme_ctx_set_engine_info(
-					ctx, GPGME_PROTOCOL_OpenPGP, file_name, home_dir);
+			if (gpgme_ctx_set_engine_info(
+					ctx, GPGME_PROTOCOL_OpenPGP, file_name, home_dir) != GPG_ERR_NO_ERROR) {
+				zend_throw_exception(
+					zend_exception_get_default(TSRMLS_C),
+					(char*) "Setting engine info failed",
+					0 TSRMLS_CC
+				);
+			}
 		}
 		gpgme_set_armor(ctx, 1);
 #if GPGME_VERSION_NUMBER >= 0x010400  /* GPGME >= 1.4.0 */
@@ -255,7 +261,7 @@ ZEND_END_ARG_INFO()
 /* }}} */
 
 /* {{{ arginfo for gnupg methods with deletekey parameter */
-ZEND_BEGIN_ARG_INFO(arginfo_gnupg_deletekey_method, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gnupg_deletekey_method, 0, 0, 1)
 	ZEND_ARG_INFO(0, kye)
 	ZEND_ARG_INFO(0, allow_secret)
 ZEND_END_ARG_INFO()
@@ -384,7 +390,7 @@ ZEND_END_ARG_INFO()
 /* }}} */
 
 /* {{{ arginfo for gnupg functions with deletekey parameter */
-ZEND_BEGIN_ARG_INFO(arginfo_gnupg_deletekey_function, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gnupg_deletekey_function, 0, 0, 2)
 	ZEND_ARG_INFO(0, res)
 	ZEND_ARG_INFO(0, kye)
 	ZEND_ARG_INFO(0, allow_secret)
@@ -686,10 +692,10 @@ gpgme_error_t passphrase_cb(
 		GNUPG_ERR("Incorrent passphrase");
 		return 1;
 	}
-	for (idx=0; idx < 16; idx++) {
+	for (idx=0; idx < 16 && uid_hint[idx] != '\0'; idx++) {
 		uid[idx] = uid_hint[idx];
 	}
-	uid[16] = '\0';
+	uid[idx] = '\0';
 	if (!PHPC_HASH_CSTR_FIND_PTR_IN_COND(
 				PHPC_THIS->signkeys, (char *)uid, passphrase)) {
 		GNUPG_ERR("no passphrase set");
@@ -726,10 +732,14 @@ gpgme_error_t passphrase_decrypt_cb (
 		GNUPG_ERR("Incorrent passphrase");
 		return 1;
 	}
-	for (idx=0; idx < 16; idx++) {
+	if (uid_hint == NULL) {
+		GNUPG_ERR("No user ID hint");
+		return 1;
+	}
+	for (idx=0; idx < 16 && uid_hint[idx] != '\0'; idx++) {
 		uid[idx] = uid_hint[idx];
 	}
-	uid[16] = '\0';
+	uid[idx] = '\0';
 	if (!PHPC_HASH_CSTR_FIND_PTR_IN_COND(
 				PHPC_THIS->decryptkeys, (char *)uid, passphrase)) {
 		GNUPG_ERR("no passphrase set");
@@ -1699,11 +1709,13 @@ PHP_FUNCTION(gnupg_decrypt)
 		return;
 	}
 	userret = gpgme_data_release_and_get_mem(out, &ret_size);
+
 	gpgme_data_release(in);
-	PHPC_CSTRL_RETVAL(userret, ret_size);
-	free(userret);
-	if (ret_size < 1) {
-		RETVAL_FALSE;
+	if (userret != NULL) {
+		PHPC_CSTRL_RETVAL(userret, ret_size);
+		free(userret);
+	} else {
+		PHPC_CSTR_EMPTY_RETVAL();
 	}
 }
 /* }}} */
